use super::handler::Handler;
use crate::config::Config;
use serenity::{all::GatewayIntents, http::Http, Client};
use std::collections::HashSet;
use tokio::task::spawn_blocking;

pub fn start_bot(config: Config) {
    if config.token != "" {
        spawn_blocking(move || {
            let rt = tokio::runtime::Handle::current();
            rt.block_on(async move {
                let local = tokio::task::LocalSet::new();
                let _ = local
                    .run_until(async move {
                        let intents = GatewayIntents::GUILD_MESSAGES
                            | GatewayIntents::DIRECT_MESSAGES
                            | GatewayIntents::MESSAGE_CONTENT
                            | GatewayIntents::GUILD_VOICE_STATES
                            | GatewayIntents::GUILDS
                            | GatewayIntents::GUILD_MEMBERS;
                        let http = Http::new(&config.token);
                        let (_owners, _bot_id) = match http.get_current_application_info().await {
                            Ok(info) => {
                                let mut owners = HashSet::new();
                                match info.owner {
                                    Some(user) => {
                                        owners.insert(user.id);
                                        println!("{}", user.global_name.unwrap())
                                    }
                                    None => {}
                                }

                                (owners, info.id)
                            }
                            Err(why) => panic!("Could not access application info: {:?}", why),
                        };
                        let mut client = Client::builder(&config.token, intents)
                            .event_handler(Handler)
                            .await
                            .expect("Err creating client");
                        if let Err(why) = client.start().await {
                            println!("Client error: {why:?}");
                        }
                    })
                    .await;
            })
        });
    }
}
